/*******************************************************************************
*                                                                              *
*   Abstract    : This file contains interrupt vector and startup code.        *
*                                                                              *
*   Functions   : Reset_Handler                                                *
*                                                                              *
*   Target      : Ambiq Micro Apollo microcontrollers                          *
*                                                                              *
*   Environment : Atollic TrueSTUDIO(R)                                        *
*                                                                              *
*   Distribution: The file is distributed "as is," without any warranty        *
*                 of any kind.                                                 *
*                                                                              *
*******************************************************************************/


  .syntax unified
  .thumb

  .global Reset_Handler
  .global InterruptVector
  .global Default_Handler

  /* Linker script definitions */
  /* start address for the initialization values of the .data section */
  .word _sidata
  /* start address for the .data section */
  .word _sdata
  /* end address for the .data section */
  .word _edata
  /* start address for the .bss section */
  .word _sbss
  /* end address for the .bss section */
  .word _ebss

/**
**===========================================================================
**  Program - Reset_Handler
**  Abstract: This code gets called after reset.
**===========================================================================
*/
  .section  .text.Reset_Handler,"ax", %progbits
  .type Reset_Handler, %function
Reset_Handler:
  /* Set stack pointer */
  ldr r0,=_estack
  mov sp, r0


  ldr     r0, =0xe000ed88
  ldr     r1,[r0]
  ldr     r2, =0xf00000
  orr     r1,r1,r2
  str     r1,[r0]


  /* Branch to SystemInit function */
  bl SystemInit

  /* Copy data initialization values */
  ldr r1,=_sidata
  ldr r2,=_sdata
  ldr r3,=_edata
  b cmpdata
CopyLoop:
  ldr r0, [r1]
  adds r1, r1, #4
  str r0, [r2]
  adds r2, r2, #4
cmpdata:
  cmp r2, r3
  blt CopyLoop

  /* Clear BSS section */
  movs r0, #0
  ldr r2,=_sbss
  ldr r3,=_ebss
  b cmpbss
ClearLoop:
  str r0, [r2]
  adds r2, r2, #4
cmpbss:
  cmp r2, r3
  blt ClearLoop

  /* Call static constructors */
  bl __libc_init_array

  /* Branch to main */
  bl main

  /* If main returns, branch to Default_Handler. */
  b Default_Handler

  .size  Reset_Handler, .-Reset_Handler

/**
**===========================================================================
**  Program - Default_Handler
**  Abstract: This code gets called when the processor receives an
**    unexpected interrupt.
**===========================================================================
*/
  .section  .text.Default_Handler,"ax", %progbits
Default_Handler:
  b  Default_Handler

  .size  Default_Handler, .-Default_Handler

/**
**===========================================================================
**  Interrupt vector table
**===========================================================================
*/
  .section .isr_vector,"a", %progbits
InterruptVector:
  .word _estack                   /* 0 - Stack pointer */
  .word Reset_Handler             /* 1 - Reset */
  .word NMI_Handler               /* 2 - NMI  */
  .word HardFault_Handler         /* 3 - Hard fault */
  .word MemManage_Handler         /* 4 - Memory management fault */
  .word BusFault_Handler          /* 5 - Bus fault */
  .word UsageFault_Handler        /* 6 - Usage fault */
  .word 0                         /* 7 - Reserved */
  .word 0                         /* 8 - Reserved */
  .word 0                         /* 9 - Reserved */
  .word 0                         /* 10 - Reserved */
  .word SVC_Handler               /* 11 - SVCall */
  .word DebugMonitor_Handler      /* 12 - Reserved for Debug */
  .word 0                         /* 13 - Reserved */
  .word PendSV_Handler            /* 14 - PendSV */
  .word SysTick_Handler           /* 15 - Systick */

  /* External Interrupts */
  .word BROWNOUT_IRQHandler       /* IRQ #0 */
  .word WDT_IRQHandler            /* IRQ #1 */
  .word CLKGEN_IRQHandler         /* IRQ #2 */
  .word VCOMP_IRQHandler          /* IRQ #3 */
  .word IOSLAVE_IRQHandler        /* IRQ #4 */
  .word IOSLAVEACC_IRQHandler     /* IRQ #5 */
  .word IOMSTR0_IRQHandler        /* IRQ #6 */
  .word IOMSTR1_IRQHandler        /* IRQ #7 */
  .word ADC_IRQHandler            /* IRQ #8 */
  .word GPIO_IRQHandler           /* IRQ #9 */
  .word CTIMER_IRQHandler         /* IRQ #10 */
  .word UART_IRQHandler           /* IRQ #11 */



/**
**===========================================================================
**  Weak interrupt handlers redirected to Default_Handler. These can be
**  overridden in user code.
**===========================================================================
*/
  .weak NMI_Handler
  .thumb_set NMI_Handler, Default_Handler

  .weak HardFault_Handler
  .thumb_set HardFault_Handler, Default_Handler

  .weak MemManage_Handler
  .thumb_set MemManage_Handler, Default_Handler

  .weak BusFault_Handler
  .thumb_set BusFault_Handler, Default_Handler

  .weak UsageFault_Handler
  .thumb_set UsageFault_Handler, Default_Handler

  .weak SVC_Handler
  .thumb_set SVC_Handler, Default_Handler

  .weak DebugMonitor_Handler
  .thumb_set DebugMonitor_Handler, Default_Handler

  .weak PendSV_Handler
  .thumb_set PendSV_Handler, Default_Handler

  .weak SysTick_Handler
  .thumb_set SysTick_Handler, Default_Handler
  
/* IRQ #0 */
  .weak BROWNOUT_IRQHandler
  .thumb_set BROWNOUT_IRQHandler, Default_Handler

/* IRQ #1 */
  .weak WDT_IRQHandler
  .thumb_set WDT_IRQHandler, Default_Handler

/* IRQ #2 */
  .weak CLKGEN_IRQHandler
  .thumb_set CLKGEN_IRQHandler, Default_Handler

/* IRQ #3 */
  .weak VCOMP_IRQHandler
  .thumb_set VCOMP_IRQHandler, Default_Handler

/* IRQ #4 */
  .weak IOSLAVE_IRQHandler
  .thumb_set IOSLAVE_IRQHandler, Default_Handler

/* IRQ #5 */
  .weak IOSLAVEACC_IRQHandler
  .thumb_set IOSLAVEACC_IRQHandler, Default_Handler

/* IRQ #6 */
  .weak IOMSTR0_IRQHandler
  .thumb_set IOMSTR0_IRQHandler, Default_Handler

/* IRQ #7 */
  .weak IOMSTR1_IRQHandler
  .thumb_set IOMSTR1_IRQHandler, Default_Handler

/* IRQ #8 */
  .weak ADC_IRQHandler
  .thumb_set ADC_IRQHandler, Default_Handler

/* IRQ #9 */
  .weak GPIO_IRQHandler
  .thumb_set GPIO_IRQHandler, Default_Handler

/* IRQ #10 */
  .weak CTIMER_IRQHandler
  .thumb_set CTIMER_IRQHandler, Default_Handler

/* IRQ #11 */
  .weak UART_IRQHandler
  .thumb_set UART_IRQHandler, Default_Handler

  .end
