/*******************************************************************************
*                                                                              * 
*   Abstract    : This file contains interrupt vector and startup code.        *
*                                                                              *
*   Functions   : Reset_Handler                                                *
*                                                                              *
*   Target      : Ambiq Micro Apollo microcontrollers                          *
*                                                                              * 
*   Environment : GNU Toolchain(Make)                                          *
*                                                                              *
*   Distribution: The file is distributed "as is," without any warranty        *
*                 of any kind.                                                 *
*                                                                              *
*******************************************************************************/


/*
// <h> Stack Configuration
//   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

	.equ    Stack_Size, 0x00000400
    .section ".stack", "w"
    .align  3
    .globl  __MCU_stack_mem
    .globl  __MCU_stack_size
__MCU_stack_mem:
    .if     Stack_Size
    .space  Stack_Size
    .endif
    .size   __MCU_stack_mem,  . - __MCU_stack_mem
    .set    __MCU_stack_size, . - __MCU_stack_mem


/*
// <h> Heap Configuration
//   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

    .equ    Heap_Size,  0x00000800
    
    .section ".heap", "w"
    .align  3
    .globl  __MCU_heap_start
    .globl  __MCU_heap_end
__MCU_heap_start:
    .if     Heap_Size
    .space  Heap_Size
    .endif
__MCU_heap_end:


/* Vector Table */

    .section ".MCU.interrupt_vector"
    .globl  __MCU_interrupt_vector_cortex_m
    .type   __MCU_interrupt_vector_cortex_m, %object

__MCU_interrupt_vector_cortex_m:
    .long   __MCU_stack                 /* Top of Stack                 */
    .long   __MCU_reset                 /* Reset Handler                */
    .long   NMI_Handler                 /* NMI Handler                  */
    .long   HardFault_Handler           /* Hard Fault Handler           */
    .long   MemManage_Handler           /* MPU Fault Handler            */
    .long   BusFault_Handler            /* Bus Fault Handler            */
    .long   UsageFault_Handler          /* Usage Fault Handler          */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   SVC_Handler                 /* SVCall Handler               */
    .long   DebugMon_Handler            /* Debug Monitor Handler        */
    .long   0                           /* Reserved                     */
    .long   PendSV_Handler              /* PendSV Handler               */
    .long   SysTick_Handler             /* SysTick Handler              */

        /* External Interrupts */

    .long   BROWNOUT_IRQHandler         /* 0: Reserved */
    .long   WDT_IRQHandler              /* 1: Reserved */
    .long   CLKGEN_IRQHandler           /* 2: CLK_GEN */
    .long   VCOMP_IRQHandler            /* 3: Voltage Comparator */
    .long   IOSLAVE_IRQHandler          /* 4: I/O Slave general */
    .long   IOSLAVEACC_IRQHandler       /* 5: I/O Slave access */
    .long   IOMSTR0_IRQHandler          /* 6: I/O Master 0 */
    .long   IOMSTR1_IRQHandler          /* 7: I/O Master 1 */
    .long   ADC_IRQHandler              /* 8: Reserved */
    .long   GPIO_IRQHandler             /* 9: GPIO  */
    .long   CTIMER_IRQHandler           /* 10: CTIMER */
    .long   UART_IRQHandler             /* 11: UART */


    .size   __MCU_interrupt_vector_cortex_m, . - __MCU_interrupt_vector_cortex_m


    .thumb


/* Reset Handler */

    .section .MCU.reset,"x",%progbits
    .thumb_func
    .globl  __MCU_reset_cortex_m
    .type   __MCU_reset_cortex_m, %function
__MCU_reset_cortex_m:
    .fnstart
.ifdef Debug_RAM
/* this routine is used for .bss sections initialization by RAM debugging "Debug_RAM = 1" */
/* Clear .bss section (Zero init) */
	MOV     R0, #0
	LDR     R1, =_sbss
	LDR     R2, =_ebss
	CMP     R1,R2
	BEQ     start_main
Loop_bss:
	CMP     R1, R2
	BHS		start_main
	STR   	R0, [R1]
	ADD		R1, #4
	BLO     Loop_bss

.else
/* this routine is used for .data and .bss sections initialization by ROM debugging "Debug_RAM = 0" */
init_data:
	LDR		R0, =_sdata
	LDR		R1, =_etext
	LDR		R2, =data_size
	MOV		R3, #0
loop_init_data:
	CMP		R3, R2
	BCS		zero_bss
    LDR		R4,[R1,#0]
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R1, #4
	ADD		R0, #4
	B		loop_init_data

/* Clear .bss section (Zero init) */

zero_bss:
	LDR		R0, =_sbss
	LDR		R2, =bss_size
	MOV		R3, #0
	MOV		R4, #0
loop_zero_bss:
	CMP		R3, R2
	BCS		start_main
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R0, #4
	B		loop_zero_bss
.endif
start_main:

  /*enable fpu begin*/
  /*; enable cp10,cp11 */
  
  
  ldr     r0, =0xe000ed88           
  ldr     r1,[r0]
  ldr     r2, =0xf00000
  orr     r1,r1,r2
  str     r1,[r0]
  
  
  /*enable fpu end*/ 
  
	LDR     R0, =SystemInit
    BLX     R0
    LDR     R0,=main
    BX      R0

    .pool
    .cantunwind
    .fnend
    .size   __MCU_reset_cortex_m,.-__MCU_reset_cortex_m

    .section ".text"

/* Exception Handlers */

    .weak   NMI_Handler
    .type   NMI_Handler, %function
NMI_Handler:
    B       .
    .size   NMI_Handler, . - NMI_Handler

    .weak   HardFault_Handler
    .type   HardFault_Handler, %function
HardFault_Handler:
    B       .
    .size   HardFault_Handler, . - HardFault_Handler

    .weak   MemManage_Handler
    .type   MemManage_Handler, %function
MemManage_Handler:
    B       .
    .size   MemManage_Handler, . - MemManage_Handler

    .weak   BusFault_Handler
    .type   BusFault_Handler, %function
BusFault_Handler:
    B       .
    .size   BusFault_Handler, . - BusFault_Handler

    .weak   UsageFault_Handler
    .type   UsageFault_Handler, %function
UsageFault_Handler:
    B       .
    .size   UsageFault_Handler, . - UsageFault_Handler

    .weak   SVC_Handler
    .type   SVC_Handler, %function
SVC_Handler:
    B       .
    .size   SVC_Handler, . - SVC_Handler

    .weak   DebugMon_Handler
    .type   DebugMon_Handler, %function
DebugMon_Handler:
    B       .
    .size   DebugMon_Handler, . - DebugMon_Handler

    .weak   PendSV_Handler
    .type   PendSV_Handler, %function
PendSV_Handler:
    B       .
    .size   PendSV_Handler, . - PendSV_Handler

    .weak   SysTick_Handler
    .type   SysTick_Handler, %function
SysTick_Handler:
    B       .
    .size   SysTick_Handler, . - SysTick_Handler


/* IRQ Handlers */

    .globl  Default_Handler
    .type   Default_Handler, %function
Default_Handler:
    B       .
    .size   Default_Handler, . - Default_Handler

    .macro  IRQ handler
    .weak   \handler
    .set    \handler, Default_Handler
    .endm

	IRQ     BROWNOUT_IRQHandler     /* 0: Reserved */
	IRQ     WDT_IRQHandler          /* 1: Reserved */
	IRQ     CLKGEN_IRQHandler		/* 2: CLK_GEN */
	IRQ     VCOMP_IRQHandler		/* 3: Voltage Comparator */
	IRQ     IOSLAVE_IRQHandler		/* 4: I/O Slave general */
	IRQ     IOSLAVEACC_IRQHandler	/* 5: I/O Slave access */
	IRQ     IOMSTR0_IRQHandler      /* 6: I/O Master 0 */
	IRQ     IOMSTR1_IRQHandler      /* 7: I/O Master 1 */
	IRQ     ADC_IRQHandler          /* 8: Reserved */
	IRQ     GPIO_IRQHandler         /* 9: GPIO */
	IRQ     CTIMER_IRQHandler       /* 10: CTIMER */
	IRQ     UART_IRQHandler         /* 11: UART */


    .end
