/******************************************************************************

Copyright (C) 2013-2017, Fujitsu Electronics Europe GmbH or a               
subsidiary of Fujitsu Electronics Europe GmbH.  All rights reserved.        
                                                                            
This software, including source code, documentation and related             
materials ("Software"), is owned by Fujitsu Electronics Europe GmbH or      
one of its subsidiaries ("Fujitsu").
                                                                            
If no EULA applies, Fujitsu hereby grants you a personal, non-exclusive,    
non-transferable license to copy, modify, and compile the                   
Software source code solely for use in connection with Fujitsu's            
integrated circuit products.  Any reproduction, modification, translation,  
compilation, or representation of this Software except as specified         
above is prohibited without the express written permission of Fujitsu.      
                                                                            
Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO                        
WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING,                        
BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED                                
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A                             
PARTICULAR PURPOSE. Fujitsu reserves the right to make                      
changes to the Software without notice. Fujitsu does not assume any         
liability arising out of the application or use of the Software or any      
product or circuit described in the Software. Fujitsu does not              
authorize its products for use in any products where a malfunction or       
failure of the Fujitsu product may reasonably be expected to result in      
significant property damage, injury or death ("High Risk Product"). By      
including Fujitsu's product in a High Risk Product, the manufacturer        
of such system or application assumes all risk of such use and in doing     
so agrees to indemnify Fujitsu against all liability.                       

 ******************************************************************************/
/******************************************************************************/
/** \file Board.h
 **
 ** A detailed description is available at 
 ** @link BoardGroup Board Configuration for ClickBeetle with Apollo1 MCU description @endlink
 **
 ** History:
 **   - 2017-12-13  V1.0  MSc  First Version

 *****************************************************************************/
#ifndef __BOARD_H__
#define __BOARD_H__

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup BoardGroup Board Configuration
 **
 ** Provided functions of Board:
 ** 
 **   
 ******************************************************************************/
//@{

/**
 ******************************************************************************    
 ** \page board_module_includes Required includes in main application
 ** \brief Following includes are required
 ** @code   
 ** #include "board.h"   
 ** @endcode
 **
 ******************************************************************************/
    
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/

//#include "skamapollobase.h"
#include "apollogpio.h"
#include "apolloiom.h"
#include "apolloctimer.h"
#include "apolloadc.h"
     
/*****************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

#define BOARD_BLUEBEETLE_I  1
//#define BOARD_BLUEBEETLE_II   1
//#define BOARD_BREAKOUTBEETLE  1
//#define BOARD_SENSORBEETLE_I  1
//#define BOARD_POWERBEETLE_I   1
    
/*((------ UNDER NDA ----------*/
//#define BOARD_BLUEBEETLE_III  1
//#define BOARD_BLUEBEETLE_IV   1
//#define BOARD_ZIGBEETLE_I     1
//#define BOARD_BLUEZBEETLE_I   1
//#define BOARD_BLUEZBEETLE_II  1
//#define BOARD_WIFIBEETLE_I    1
//#define BOARD_MULTIBEETLE_I   1
/*-------- UNDER NDA --------))*/
    
#define LED_RED             44
#define LED_GREEN           46
#define LED_BLUE            48
#define BOARD_SW1           4

#define BOARD_A0            12
#define BOARD_A1            13
#define BOARD_A2            14
#define BOARD_A3            29

#define BOARD_UART_RX_PIN   1
#define BOARD_UART_TX_PIN   0
#define BOARD_UART_RX_FUNC  2
#define BOARD_UART_TX_FUNC  2
#define BOARD_UART          UART    
#define BOARD_UART_NO       0
    
#define BOARD_I2C_SCL_PIN   8
#define BOARD_I2C_SDA_PIN   9     
#define BOARD_I2C_SCL_FUNC  0
#define BOARD_I2C_SDA_FUNC  0 
#define BOARD_I2C           IOMSTR1
#define BOARD_I2C_NO        1

#define BOARD_SPI_SCK_PIN   5
#define BOARD_SPI_MISO_PIN  6   
#define BOARD_SPI_MOSI_PIN  7 
#define BOARD_SPI_SCK_FUNC  1
#define BOARD_SPI_MISO_FUNC 1 
#define BOARD_SPI_MOSI_FUNC 1
#define BOARD_SPI           IOMSTR0
#define BOARD_SPI_CS0       10
#define BOARD_SPI_CS1       34
#define BOARD_SPI_CS2       18
#define BOARD_SPI_CS3       33    
#define BOARD_SPI_NO        0

#define BOARD_RF_PWR        3

//BlueBeetle I
#define NRF8001_PWR         BOARD_RF_PWR    
#define NRF8001_MISO        BOARD_SPI_MISO_PIN
#define NRF8001_MOSI        BOARD_SPI_MOSI_PIN
#define NRF8001_SCK         BOARD_SPI_SCK_PIN
#define NRF8001_CS          BOARD_SPI_CS0
#define NRF8001_RDY         28
#define NRF8001_RST         25
#define NRF8001_ACT         24    

//BlueBeetle II    
#define EM9304_PWR         BOARD_RF_PWR    
#define EM9304_MISO        BOARD_SPI_MISO_PIN
#define EM9304_MOSI        BOARD_SPI_MOSI_PIN
#define EM9304_SCK         BOARD_SPI_SCK_PIN
#define EM9304_CS          BOARD_SPI_CS0
#define EM9304_RDY         28
#define EM9304_GPIO5       25
#define EM9304_ACT         24    

//ZigBeetle
#define GP712_PWR         BOARD_RF_PWR    
#define GP712_MISO        BOARD_SPI_MISO_PIN
#define GP712_MOSI        BOARD_SPI_MOSI_PIN
#define GP712_SCK         BOARD_SPI_SCK_PIN
#define GP712_CS          BOARD_SPI_CS0
#define GP712_INT         24
#define GP712_RST         25
#define GP712_WKUP        28    
    
/*****************************************************************************/
/* Global type definitions ('typedef')                                        */
/*****************************************************************************/


/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/



/*****************************************************************************/
/* Global function prototypes ('extern', definition in C source)             */
/*****************************************************************************/


#ifdef __cplusplus
}
#endif

void Board_Init(void);

//@} // BoardGroup

#endif /*__BOARD_H__*/

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

